@@ -16,13 +16,17 @@
 
 	drv <- getOption("sqldf.driver")
 	drv <- if (is.null(drv) || drv == "") {
-		if ("package:RpgSQL" %in% search()) { 
+
+		if ("package:Rostgresql" %in% search()) { 
+			"PostgreSQL"
+		} else if ("package:RpgSQL" %in% search()) { 
 			"pgSQL"
 		} else if ("package:RMySQL" %in% search()) { 
 			"MySQL" 
 		} else if ("package:RH2" %in% search()) { 
 			"H2" 
 		} else "SQLite"
+
 	} else if (!tolower(drv) %in% c("pgsql", "mysql", "h2")) {
 		"SQLite"
 	}
